package sun.misc;

import java.util.HashMap;
import java.util.List;
import java.io.IOException;
import java.util.ArrayList;
import java.io.BufferedReader;
import java.io.FileReader;
import java.io.File;
import java.util.Map;

public class MetaIndex
{
    private static volatile Map<File, MetaIndex> jarMap;
    private String[] contents;
    private boolean isClassOnlyJar;
    
    public static MetaIndex forJar(File file) {
        return (MetaIndex)getJarMap().get(file);
    }
    
    public static synchronized void registerDirectory(File canonicalFile) {
        File file = new File(canonicalFile, "meta-index");
        if (file.exists()) {
            try {
                BufferedReader bufferedReader = new BufferedReader(new FileReader(file));
                String substring = null;
                boolean n = false;
                ArrayList<String> list = new ArrayList();
                Map<File, MetaIndex> jarMap = getJarMap();
                canonicalFile = canonicalFile.getCanonicalFile();
                String line = bufferedReader.readLine();
                if (line == null || !line.equals("% VERSION 2")) {
                    bufferedReader.close();
                    return;
                }
                String line2;
                while ((line2 = bufferedReader.readLine()) != null) {
                    switch (line2.charAt(0)) {
                        case '!':
                        case '#':
                        case '@': {
                            if (substring != null && list.size() > 0) {
                                jarMap.put(new File(canonicalFile, substring), new MetaIndex(list, (boolean)n));
                                list.clear();
                            }
                            substring = line2.substring(2);
                            if (line2.charAt(0) == '!') {
                                n = true;
                            }
                            else {
                                if (n == 0) {
                                    continue;
                                }
                                n = false;
                            }
                        }
                        case '%': {
                            continue;
                        }
                        default: {
                            list.add(line2);
                            continue;
                        }
                    }
                }
                if (substring != null && list.size() > 0) {
                    jarMap.put(new File(canonicalFile, substring), new MetaIndex(list, (boolean)n));
                }
                bufferedReader.close();
            }
            catch (IOException ex) {}
        }
    }
    
    public boolean mayContain(String s) {
        if (this.isClassOnlyJar && !s.endsWith(".class")) {
            return false;
        }
        String[] contents = this.contents;
        int i = 0;
        while (i < contents.length) {
            if (s.startsWith(contents[i])) {
                return true;
            }
            ++i;
        }
        return false;
    }
    
    private MetaIndex(List<String> list, boolean isClassOnlyJar) throws IllegalArgumentException {
        if (list == null) {
            throw new IllegalArgumentException();
        }
        this.contents = (String[])list.toArray(new String[0]);
        this.isClassOnlyJar = isClassOnlyJar;
        return;
    }
    
    private static Map<File, MetaIndex> getJarMap() {
        if (MetaIndex.jarMap == null) {
            synchronized (MetaIndex.class) {
                if (MetaIndex.jarMap == null) {
                    MetaIndex.jarMap = new HashMap();
                }
            }
        }
        assert MetaIndex.jarMap != null;
        return MetaIndex.jarMap;
    }
}
